<?php

namespace Drupal\views_add_after_nth_rows\Plugin\views\field;

use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Drupal\views\Plugin\views\display\DisplayPluginBase;
use Drupal\views\ViewExecutable;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Render\ViewsRenderPipelineMarkup;
use Drupal\Component\Utility\Xss;

/**
 * A handler to provide a field that is added by the administrator.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("views_add_after_nth_rows_handler_field")
 */
class viewsAddAfterNthRowsHandler extends FieldPluginBase {
  /**
   * The current display.
   *
   * @var string
   *   The current display of the view.
   */
  protected $currentDisplay;

  /**
   * {@inheritdoc}
   */
  public function init(ViewExecutable $view, DisplayPluginBase $display, array &$options = NULL) {
    parent::init($view, $display, $options);
    $this->currentDisplay = $view->current_display;
  }

  /**
   * {@inheritdoc}
   */
  public function usesGroupBy() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Do nothing -- to override the parent query.
  }


  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    // Override the alter text option to always alter the text.
    $options['alter']['contains']['alter_text'] = ['default' => TRUE];
    $options['hide_alter_empty'] = ['default' => FALSE];
    $options['nthrow'] = ['default' => 1];

    return $options;
  }

  /**
   * Provide the options form.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {

    // Nthrow option.
    $form['nthrow'] = [
      '#type' => 'number',
      '#title' => $this->t('Display every how many row(s)?'),
      '#weight' => -999,
      '#default_value' => $this->options['nthrow'],
      '#description' => $this->t('Display this field once every number of rows specified here.'),
    ];

    // Remove the checkbox
    unset($form['alter']['alter_text']);
    unset($form['alter']['text']['#states']);
    unset($form['alter']['help']['#states']);
    $form['#pre_render'][] = [$this, 'preRenderNthRowCustomForm'];
    parent::buildOptionsForm($form, $form_state);
  }

  /**
   * Render this field as user-defined altered text.
   */
  protected function renderAltered($alter, $tokens) {
    $return = NULL;
    if ((int) ($this->view->row_index + 1) % (int) $this->options['nthrow'] === 0) {
      $text = ViewsRenderPipelineMarkup::create(Xss::filterAdmin($alter['text']));
      $return = $this->viewsTokenReplace($text, $tokens);
    }
    return $return;
  }

  /**
   * Prerender function to move the textarea to the top of a form.
   *
   * @param array $form
   *   The form build array.
   *
   * @return array
   *   The modified form build array.
   */
  public function preRenderNthRowCustomForm($form) {
    $form['text'] = $form['alter']['text'];
    $form['help'] = $form['alter']['help'];
    unset($form['alter']['text']);
    unset($form['alter']['help']);

    return $form;
  }
}
